@@ -195,6 +195,12 @@ impl Address {
         Ok(())
     }
 
+    pub fn recompute_last_pages(&self) -> PERes<()> {
+        let mut segments = self.segments.write()?;
+        segments.recompute_last_pages(&self.allocator)?;
+        Ok(())
+    }
+
     pub fn check_persistent_records(
         &self,
         records: &[(SegmentId, RecRef, u16)],
@@ -271,7 +277,6 @@ impl Address {
                 let n_page = self.get_or_insert_mut(&mut pages, new_page.page)?;
                 n_page.set_prev(new_page.previous)?;
                 n_page.set_segment_id(new_page.segment)?;
-                segments.set_segment_last_page(new_page.segment, new_page.page);
             }
         }
         for insert in inserts {
@@ -384,6 +389,11 @@ impl Address {
         Ok(())
     }
 
+    pub fn flush_segments(&self) -> PERes<()> {
+        let mut segments = self.segments.write()?;
+        segments.flush_segments(&self.allocator)
+    }
+
     pub fn exists_segment(&self, segment: &str) -> PERes<bool> {
         Ok(self.segments.read()?.has_segment(segment))
     }
