@@ -1,8 +1,8 @@
 use crate::{
     error::PERes,
     io::{
-        read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat, ReadFormat,
-        WriteFormat,
+        read_u64, write_u64, ArcSliceRead, InfallibleRead, InfallibleReadFormat, InfallibleWrite,
+        InfallibleWriteFormat, ReadFormat, WriteFormat,
     },
 };
 use std::{
@@ -77,6 +77,7 @@ pub struct ReadPage {
     exp: u8,
     pos: usize,
 }
+
 impl ReadPage {
     pub fn new(buff: Arc<Vec<u8>>, pos: usize, index: u64, exp: u8) -> ReadPage {
         ReadPage { buff, index, exp, pos }
@@ -93,6 +94,9 @@ impl ReadPage {
         debug_assert!(final_len < self.buff.len() - 1);
         &self.buff[self.pos..final_len]
     }
+    pub fn arc_slice(&self, len: usize) -> ArcSliceRead {
+        ArcSliceRead::new(self.buff.clone(), self.pos, self.pos + len)
+    }
 }
 
 impl Read for ReadPage {
@@ -108,6 +112,14 @@ impl InfallibleRead for ReadPage {
     fn read_exact(&mut self, buf: &mut [u8]) {
         Read::read_exact(self, buf).expect("in memory buff never fail");
     }
+
+    fn read_slice(&mut self, size: usize) -> ArcSliceRead {
+        // There is a -1 because the last byte is metadata
+        debug_assert!(self.buff.len() - 1 > self.pos + size);
+        let read = ArcSliceRead::new(self.buff.clone(), self.pos, size);
+        self.pos += size;
+        read
+    }
 }
 
 impl PageOps for ReadPage {
@@ -173,6 +185,14 @@ impl Page {
     pub fn new(buff: Vec<u8>, pos: usize, index: u64, exp: u8) -> Page {
         Page { buff, index, exp, pos }
     }
+    pub fn new_alloc(index: u64, exp: u8) -> Page {
+        let size = 1 << exp;
+        let mut buff = vec![0; size];
+        buff[0] = exp;
+        buff[1] = free_flag_set(0, false);
+        buff[size - 1] = exp;
+        Self::new(buff, 2, index, exp)
+    }
     pub fn clone_resetted(&self) -> Page {
         let mut page = self.clone();
         page.pos = 2;
@@ -182,6 +202,7 @@ impl Page {
 
 impl Read for Page {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
+        // Last byte of the page is metadata so skip it
         let len = self.buff.len() - 1;
         let amt = cmp::min(self.pos, len);
         let read = Read::read(&mut &self.buff[(amt as usize)..len], buf)?;
@@ -194,11 +215,18 @@ impl InfallibleRead for Page {
     fn read_exact(&mut self, buf: &mut [u8]) {
         Read::read_exact(self, buf).expect("in memory buff never fail");
     }
+
+    fn read_slice(&mut self, size: usize) -> ArcSliceRead {
+        let mut slice: Vec<u8> = vec![0; size];
+        InfallibleRead::read_exact(self, &mut slice);
+        ArcSliceRead::new_vec(slice)
+    }
 }
 
 impl Write for Page {
     fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
         let pre = self.pos;
+        // Last byte of the page is metadata so skip it
         let len = self.buff.len() - 1;
         if pre + (*buf).len() > len {
             panic!(
@@ -305,16 +333,16 @@ fn load_page<T: Read + Seek>(fl: &mut T, page: u64) -> PERes<ReadPage> {
     // add 2 to skip the metadata
     fl.seek(SeekFrom::Start(page))?;
     let exp = fl.read_u8()?;
-    let size = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
-    let mut ve = vec![0u8; size as usize];
+    let size = 1 << exp; //EXP - (size_exp+size_mitigator);
+    let mut ve = vec![0u8; size];
     ve[0] = exp;
-    fl.read_exact(&mut ve[1..size as usize])?;
+    fl.read_exact(&mut ve[1..size])?;
     Ok(ReadPage::new(Arc::new(ve), 2, page, exp))
 }
 
 fn create_page_raw<T: Write + Seek>(fl: &mut T, exp: u8) -> PERes<u64> {
-    let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
-    let ve = vec![0u8; size as usize];
+    let size = 1 << exp; //EXP - (size_exp+size_mitigator);
+    let ve = vec![0u8; size];
     let offset = fl.seek(SeekFrom::End(0))?;
     fl.write_all(&ve)?;
     Ok(offset)
@@ -325,9 +353,9 @@ fn load_page_raw<T: Read + Seek>(fl: &mut T, page: u64, size_exp: u8) -> PERes<P
     let size;
     {
         fl.seek(SeekFrom::Start(page))?;
-        size = (1 << size_exp) as u64; //EXP - (size_exp+size_mitigator);
-        ve = vec![0u8; size as usize];
-        fl.read_exact(&mut ve[0..size as usize])?;
+        size = 1 << size_exp; //EXP - (size_exp+size_mitigator);
+        ve = vec![0u8; size];
+        fl.read_exact(&mut ve[0..size])?;
     }
     Ok(Page::new(ve, 0, page, size_exp))
 }
@@ -339,10 +367,10 @@ fn flush_page<T: Write + Seek>(fl: &mut T, page: &Page) -> PERes<()> {
 }
 
 fn create_page<T: Write + Seek>(fl: &mut T, exp: u8) -> PERes<Page> {
-    let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
-    let mut ve = vec![0u8; size as usize];
+    let size = 1 << exp; //EXP - (size_exp+size_mitigator);
+    let mut ve = vec![0u8; size];
     ve[0] = exp;
-    ve[size as usize - 1] = exp;
+    ve[size - 1] = exp;
     let offset = fl.seek(SeekFrom::End(0))?;
     fl.write_all(&ve)?;
     Ok(Page::new(ve, 2, offset, exp))
@@ -526,10 +554,10 @@ impl Device for DiscRef {
         use std::os::unix::prelude::FileExt;
         let mut exp = [0u8];
         self.file.read_exact_at(&mut exp, page)?;
-        let size = (1 << exp[0]) as u64; //EXP - (size_exp+size_mitigator);
-        let mut ve = vec![0u8; size as usize];
+        let size = 1 << exp[0]; //EXP - (size_exp+size_mitigator);
+        let mut ve = vec![0u8; size];
         ve[0] = exp[0];
-        self.file.read_exact_at(&mut ve[1..size as usize], page + 1)?;
+        self.file.read_exact_at(&mut ve[1..size], page + 1)?;
         // add 2 to skip the metadata
         Ok(ReadPage::new(Arc::new(ve), 2, page, exp[0]))
     }
@@ -545,8 +573,8 @@ impl Device for DiscRef {
 
     fn load_page_raw(&self, page: u64, size_exp: u8) -> PERes<Page> {
         use std::os::unix::prelude::FileExt;
-        let size = (1 << size_exp) as u64; //EXP - (size_exp+size_mitigator);
-        let mut ve = vec![0u8; size as usize];
+        let size = 1 << size_exp; //EXP - (size_exp+size_mitigator);
+        let mut ve = vec![0u8; size];
         self.file.read_exact_at(&mut ve, page)?;
         Ok(Page::new(ve, 0, page, size_exp))
     }
@@ -559,20 +587,20 @@ impl Device for DiscRef {
 
     fn create_page_raw(&self, exp: u8) -> PERes<u64> {
         use std::os::unix::prelude::FileExt;
-        let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
-        let offset = self.create_page_offset(size)?;
-        let ve = vec![0u8; size as usize];
+        let size = (1 << exp) as usize; //EXP - (size_exp+size_mitigator);
+        let offset = self.create_page_offset(size as u64)?;
+        let ve = vec![0u8; size];
         self.file.write_all_at(&ve, offset)?; //exp
         Ok(offset)
     }
 
     fn create_page(&self, exp: u8) -> PERes<Page> {
         use std::os::unix::prelude::FileExt;
-        let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
-        let offset = self.create_page_offset(size)?;
-        let mut ve = vec![0u8; size as usize];
+        let size = (1 << exp) as usize; //EXP - (size_exp+size_mitigator);
+        let offset = self.create_page_offset(size as u64)?;
+        let mut ve = vec![0u8; size];
         ve[0] = exp;
-        ve[size as usize - 1] = exp;
+        ve[size - 1] = exp;
         self.file.write_all_at(&ve, offset)?; //exp
         Ok(Page::new(ve, 2, offset, exp))
     }
@@ -782,7 +810,7 @@ impl Device for MemRef {
 mod tests {
     use super::{Device, DiscRef, MemRef, PageOps, UpdateList};
     use crate::error::PERes;
-    use byteorder::{ReadBytesExt, WriteBytesExt};
+    use std::io::{Read, Write};
     use tempfile::Builder;
 
     fn temp_disc_ref(name: &str) -> DiscRef {
@@ -825,13 +853,14 @@ mod tests {
         let page = device.create_page(5).unwrap().get_index();
         {
             let pg = &mut device.load_page(page).unwrap().clone_write();
-            pg.write_u8(10).unwrap();
+            pg.write_all(&[10]).unwrap();
             device.flush_page(pg).unwrap();
         }
         {
             let pg = &mut device.load_page(page).unwrap();
-            let va = pg.read_u8().unwrap();
-            assert_eq!(va, 10);
+            let mut data = [0; 1];
+            pg.read_exact(&mut data).unwrap();
+            assert_eq!(data[0], 10);
             let sz = pg.get_size_exp();
             assert_eq!(sz, 5);
         }
