@@ -3,16 +3,18 @@ use crate::{
     error::{CreateIndexError, DropIndexError, IndexChangeError, IndexError, IndexOpsError, PERes, TimeoutError},
     id::{index_id_to_segment_id_meta, IndexId, PersyId, RecRef},
     index::{
+        bytevec::ByteVec,
         keeper::{Extractor, IndexSegmentKeeper, IndexSegmentKeeperTx},
         serialization::IndexSerialization,
+        string_wrapper::StringWrapper,
     },
-    io::{InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
+    io::{ArcSliceRead, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
     locks::RwLockManager,
     persy::PersyImpl,
     snapshots::SnapshotId,
     transaction_impl::TransactionImpl,
 };
-use std::{fmt::Display, io::Cursor, ops::Deref, str, sync::Arc};
+use std::{fmt::Display, str, sync::Arc};
 
 /// Enum of all the possible Key or Value types for indexes
 #[derive(Clone)]
@@ -57,16 +59,86 @@ impl From<u8> for IndexTypeId {
     }
 }
 
-pub trait WrapperType<T>: Clone + From<T> + Extractor + Ord {
-    fn value(self) -> T;
-}
+pub trait IndexType: IndexTypeWrap + Clone {}
+
 /// Trait implemented by all supported types in the index
-pub trait IndexType: Display + IndexOrd + Clone + IndexSerialization {
-    type Wrapper: WrapperType<Self>;
+pub trait IndexTypeInternal: Display + IndexOrd + Clone + Extractor + IndexSerialization + IndexTypeUnwrap {
     fn get_id() -> u8;
     fn get_type_id() -> IndexTypeId;
 }
 
+pub trait IndexTypeUnwrap {
+    type Wrapped;
+    fn unwrap(self) -> Self::Wrapped;
+}
+
+pub trait IndexTypeWrap: Sized {
+    type Wrapper: IndexTypeInternal + IndexTypeUnwrap<Wrapped = Self>;
+    fn wrap(self) -> Self::Wrapper
+    where
+        Self: Sized;
+}
+
+macro_rules! impl_wrapper_trait_self {
+    ($($t:ty),+,) => {
+        $(
+        impl IndexTypeUnwrap for $t {
+            type Wrapped = $t;
+            fn unwrap(self) -> $t {
+                self
+            }
+        }
+        impl IndexTypeWrap for $t {
+            type Wrapper = $t;
+            fn wrap(self) -> $t {
+                self
+            }
+        }
+
+        )+
+    }
+}
+impl_wrapper_trait_self!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, f32, f64, PersyId, ByteVec,);
+
+macro_rules! impl_index_type {
+    ($t:ty, $v:expr,$v1:ident) => {
+        impl IndexTypeInternal for $t {
+            fn get_id() -> u8 {
+                $v
+            }
+            fn get_type_id() -> IndexTypeId {
+                IndexTypeId::$v1
+            }
+        }
+        impl IndexType for $t {}
+    };
+}
+
+impl_index_type!(u8, 1, U8);
+impl_index_type!(u16, 2, U16);
+impl_index_type!(u32, 3, U32);
+impl_index_type!(u64, 4, U64);
+impl_index_type!(u128, 14, U128);
+impl_index_type!(i8, 5, I8);
+impl_index_type!(i16, 6, I16);
+impl_index_type!(i32, 7, I32);
+impl_index_type!(i64, 8, I64);
+impl_index_type!(i128, 15, I128);
+impl_index_type!(f32, 9, F32W);
+impl_index_type!(f64, 10, F64W);
+impl_index_type!(PersyId, 13, PersyId);
+impl_index_type!(ByteVec, 16, ByteVec);
+
+impl IndexType for String {}
+impl IndexTypeInternal for StringWrapper {
+    fn get_id() -> u8 {
+        12
+    }
+    fn get_type_id() -> crate::IndexTypeId {
+        IndexTypeId::String
+    }
+}
+
 pub trait IndexOrd {
     fn cmp(&self, other: &Self) -> std::cmp::Ordering;
 }
@@ -83,7 +155,7 @@ macro_rules! impl_index_ord {
     };
 }
 
-impl_index_ord!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, String, PersyId, ByteVec);
+impl_index_ord!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, PersyId, ByteVec);
 
 impl IndexOrd for f32 {
     fn cmp(&self, other: &Self) -> std::cmp::Ordering {
@@ -117,46 +189,6 @@ impl IndexOrd for f64 {
     }
 }
 
-/// Wrapper for `Vec<u8>` for use it in index keys or values
-#[derive(Debug, PartialOrd, PartialEq, Clone, Ord, Eq)]
-pub struct ByteVec(Vec<u8>);
-
-impl ByteVec {
-    pub fn new(value: Vec<u8>) -> Self {
-        ByteVec(value)
-    }
-}
-impl Deref for ByteVec {
-    type Target = [u8];
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
-impl From<&[u8]> for ByteVec {
-    fn from(f: &[u8]) -> ByteVec {
-        ByteVec(f.to_vec())
-    }
-}
-
-impl From<Vec<u8>> for ByteVec {
-    fn from(f: Vec<u8>) -> ByteVec {
-        ByteVec(f)
-    }
-}
-
-impl From<ByteVec> for Vec<u8> {
-    fn from(f: ByteVec) -> Vec<u8> {
-        f.0
-    }
-}
-
-impl Display for ByteVec {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{:?}", &self.0)
-    }
-}
-
 pub const INDEX_META_PREFIX: &str = "+_M";
 pub const INDEX_DATA_PREFIX: &str = "+_D";
 
@@ -279,7 +311,7 @@ impl IndexConfig {
         })
     }
 
-    pub fn check<K: IndexType, V: IndexType>(&self) -> Result<(), IndexOpsError> {
+    pub fn check<K: IndexTypeInternal, V: IndexTypeInternal>(&self) -> Result<(), IndexOpsError> {
         if self.key_type != K::get_id() {
             Err(IndexOpsError::IndexTypeMismatch("key type".into()))
         } else if self.value_type != V::get_id() {
@@ -289,7 +321,7 @@ impl IndexConfig {
         }
     }
     pub fn get_root(&self) -> Option<RecRef> {
-        self.root.clone()
+        self.root
     }
 }
 
@@ -310,8 +342,8 @@ impl Indexes {
         value_mode: ValueMode,
     ) -> Result<(), CreateIndexError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         debug_assert!(min <= max / 2);
         let segment_name_meta = format_segment_name_meta(name);
@@ -350,7 +382,7 @@ impl Indexes {
         let segment_meta = index_id_to_segment_id_meta(index_id);
         let mut scan = p.scan_tx(tx, segment_meta)?;
         let (id, mut config) = if let Some((rid, content, _)) = scan.next(p, tx) {
-            (rid, IndexConfig::deserialize(&mut Cursor::new(content))?)
+            (rid, IndexConfig::deserialize(&mut ArcSliceRead::new_vec(content))?)
         } else {
             return Err(IndexChangeError::IndexNotFound);
         };
@@ -374,7 +406,7 @@ impl Indexes {
         let mut scan = p.scan_tx(tx, segment_meta)?;
         if let Some((_, content, version)) = scan.next(p, tx) {
             scan.release(p)?;
-            Ok((IndexConfig::deserialize(&mut Cursor::new(content))?, version))
+            Ok((IndexConfig::deserialize(&mut ArcSliceRead::new_vec(content))?, version))
         } else {
             scan.release(p)?;
             Err(IndexError::IndexNotFound)
@@ -397,11 +429,11 @@ impl Indexes {
         let segment_meta = index_id_to_segment_id_meta(index_id);
         p.scan_snapshot_index(segment_meta, snapshot_id)?
             .next(p)
-            .map(|(_, content)| IndexConfig::deserialize(&mut Cursor::new(content)).map_err(IndexError::from))
+            .map(|(_, content)| IndexConfig::deserialize(&mut ArcSliceRead::new_vec(content)).map_err(IndexError::from))
             .unwrap_or(Err(IndexError::IndexNotFound))
     }
 
-    pub fn get_index_keeper<'a, K: IndexType, V: IndexType>(
+    pub fn get_index_keeper<'a, K: IndexTypeInternal, V: IndexTypeInternal>(
         p: &'a PersyImpl,
         snapshot: SnapshotId,
         index_id: &IndexId,
@@ -418,7 +450,7 @@ impl Indexes {
         ))
     }
 
-    pub fn get_index_keeper_tx<'a, K: IndexType, V: IndexType>(
+    pub fn get_index_keeper_tx<'a, K: IndexTypeInternal, V: IndexTypeInternal>(
         p: &'a PersyImpl,
         tx: &'a mut TransactionImpl,
         index_id: &IndexId,
@@ -438,7 +470,7 @@ impl Indexes {
         ))
     }
 
-    pub fn check_index<K: IndexType, V: IndexType>(
+    pub fn check_index<K: IndexTypeInternal, V: IndexTypeInternal>(
         p: &PersyImpl,
         tx: &mut TransactionImpl,
         index_id: &IndexId,
@@ -478,8 +510,9 @@ impl Indexes {
 
 #[cfg(test)]
 mod tests {
+    use crate::io::ArcSliceRead;
+
     use super::{IndexConfig, ValueMode};
-    use std::io::Cursor;
 
     #[test]
     fn test_config_ser_des() {
@@ -495,7 +528,7 @@ mod tests {
 
         let mut buff = Vec::new();
         cfg.serialize(&mut buff);
-        let read = IndexConfig::deserialize(&mut Cursor::new(&mut buff)).expect("deserialization works");
+        let read = IndexConfig::deserialize(&mut ArcSliceRead::new_vec(buff)).expect("deserialization works");
         assert_eq!(cfg, read);
     }
 }
