@@ -1,11 +1,12 @@
-use crate::error::PERes;
-use byteorder::{BigEndian, ByteOrder, ReadBytesExt, WriteBytesExt};
-use std::io::{Cursor, Read, Write};
+use crate::{error::PERes, index::string_wrapper::StringWrapper, ByteVec};
+use std::io::{Read, Write};
+use std::sync::Arc;
 use unsigned_varint::{encode, io};
 use zigzag::ZigZag;
 
 pub trait InfallibleRead {
     fn read_exact(&mut self, buf: &mut [u8]);
+    fn read_slice(&mut self, len: usize) -> ArcSliceRead;
 }
 
 pub trait InfallibleWrite {
@@ -296,87 +297,149 @@ pub(crate) trait InfallibleReadFormat: InfallibleRead {
 
 impl<R: InfallibleRead + ?Sized> InfallibleReadFormat for R {}
 
-impl<T> InfallibleRead for Cursor<T>
-where
-    T: AsRef<[u8]>,
-{
+pub struct ArcSliceRead {
+    buff: Arc<Vec<u8>>,
+    cursor: usize,
+    limit: usize,
+}
+
+impl ArcSliceRead {
+    pub(crate) fn new(buff: Arc<Vec<u8>>, cursor: usize, limit: usize) -> Self {
+        Self { buff, cursor, limit }
+    }
+
+    pub(crate) fn new_vec(buff: Vec<u8>) -> Self {
+        let limit = buff.len();
+        Self {
+            buff: Arc::new(buff),
+            cursor: 0,
+            limit,
+        }
+    }
+
+    pub(crate) fn to_vec(self) -> Vec<u8> {
+        let mut buf = vec![0; self.limit - self.cursor];
+        Read::read(&mut &self.buff[(self.cursor as usize)..self.limit], &mut buf).unwrap();
+        buf
+    }
+
+    #[allow(unused)]
+    pub(crate) fn cursor(&self) -> usize {
+        self.cursor
+    }
+
+    pub(crate) fn to_byte_vec(self) -> ByteVec {
+        ByteVec::new_slice(self.buff, self.cursor, self.limit)
+    }
+
+    pub(crate) fn to_string_wrapper(self) -> StringWrapper {
+        StringWrapper::new_slice(self.buff, self.cursor, self.limit)
+    }
+}
+
+impl Read for ArcSliceRead {
+    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
+        let len = self.limit;
+        let amt = std::cmp::min(self.cursor, len);
+        let read = Read::read(&mut &self.buff[(amt as usize)..len], buf)?;
+        self.cursor += read;
+        Ok(read)
+    }
+}
+
+impl InfallibleRead for ArcSliceRead {
     fn read_exact(&mut self, buf: &mut [u8]) {
         Read::read_exact(self, buf).expect("never fail in a in memory buffer");
     }
+
+    fn read_slice(&mut self, size: usize) -> ArcSliceRead {
+        debug_assert!(self.limit >= self.cursor + size);
+        let read = ArcSliceRead::new(self.buff.clone(), self.cursor, size);
+        self.cursor += size;
+        read
+    }
 }
 
 pub(crate) fn read_u16(buf: &[u8]) -> u16 {
-    BigEndian::read_u16(buf)
+    let mut value = [0; 2];
+    value[0] = buf[0];
+    value[1] = buf[1];
+    u16::from_be_bytes(value)
 }
 pub(crate) fn write_u16(buf: &mut [u8], val: u16) {
-    BigEndian::write_u16(buf, val)
+    let bv = val.to_be_bytes();
+    buf[0] = bv[0];
+    buf[1] = bv[1];
 }
 pub(crate) fn read_u64(buf: &[u8]) -> u64 {
-    BigEndian::read_u64(buf)
+    let mut value = [0; 8];
+    value[..8].clone_from_slice(&buf[..8]);
+    u64::from_be_bytes(value)
 }
 pub(crate) fn write_u64(buf: &mut [u8], val: u64) {
-    BigEndian::write_u64(buf, val)
+    let value = val.to_be_bytes();
+    buf[..8].clone_from_slice(&value[..8]);
 }
 
 pub(crate) trait WriteFormat: Write {
     #[inline]
     fn write_u8(&mut self, value: u8) -> PERes<()> {
-        Ok(WriteBytesExt::write_u8(self, value)?)
+        Ok(self.write_all(&[value])?)
     }
 
     #[inline]
     fn write_u16(&mut self, value: u16) -> PERes<()> {
-        Ok(WriteBytesExt::write_u16::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_u32(&mut self, value: u32) -> PERes<()> {
-        Ok(WriteBytesExt::write_u32::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_u64(&mut self, value: u64) -> PERes<()> {
-        Ok(WriteBytesExt::write_u64::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_u128(&mut self, value: u128) -> PERes<()> {
-        Ok(WriteBytesExt::write_u128::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_i8(&mut self, value: i8) -> PERes<()> {
-        Ok(WriteBytesExt::write_i8(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_i16(&mut self, value: i16) -> PERes<()> {
-        Ok(WriteBytesExt::write_i16::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_i32(&mut self, value: i32) -> PERes<()> {
-        Ok(WriteBytesExt::write_i32::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_i64(&mut self, value: i64) -> PERes<()> {
-        Ok(WriteBytesExt::write_i64::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_i128(&mut self, value: i128) -> PERes<()> {
-        Ok(WriteBytesExt::write_i128::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_f32(&mut self, value: f32) -> PERes<()> {
-        Ok(WriteBytesExt::write_f32::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 
     #[inline]
     fn write_f64(&mut self, value: f64) -> PERes<()> {
-        Ok(WriteBytesExt::write_f64::<BigEndian>(self, value)?)
+        Ok(self.write_all(&value.to_be_bytes())?)
     }
 }
 
@@ -385,62 +448,86 @@ impl<W: Write + ?Sized> WriteFormat for W {}
 pub(crate) trait ReadFormat: Read {
     #[inline]
     fn read_u8(&mut self) -> PERes<u8> {
-        Ok(ReadBytesExt::read_u8(self)?)
+        let mut buf = [0; 1];
+        self.read_exact(&mut buf)?;
+        Ok(buf[0])
     }
 
     #[inline]
     fn read_u16(&mut self) -> PERes<u16> {
-        Ok(ReadBytesExt::read_u16::<BigEndian>(self)?)
+        let mut buf = [0; 2];
+        self.read_exact(&mut buf)?;
+        Ok(u16::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_u32(&mut self) -> PERes<u32> {
-        Ok(ReadBytesExt::read_u32::<BigEndian>(self)?)
+        let mut buf = [0; 4];
+        self.read_exact(&mut buf)?;
+        Ok(u32::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_u64(&mut self) -> PERes<u64> {
-        Ok(ReadBytesExt::read_u64::<BigEndian>(self)?)
+        let mut buf = [0; 8];
+        self.read_exact(&mut buf)?;
+        Ok(u64::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_u128(&mut self) -> PERes<u128> {
-        Ok(ReadBytesExt::read_u128::<BigEndian>(self)?)
+        let mut buf = [0; 16];
+        self.read_exact(&mut buf)?;
+        Ok(u128::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_i8(&mut self) -> PERes<i8> {
-        Ok(ReadBytesExt::read_i8(self)?)
+        let mut buf = [0; 1];
+        self.read_exact(&mut buf)?;
+        Ok(buf[0] as i8)
     }
 
     #[inline]
     fn read_i16(&mut self) -> PERes<i16> {
-        Ok(ReadBytesExt::read_i16::<BigEndian>(self)?)
+        let mut buf = [0; 2];
+        self.read_exact(&mut buf)?;
+        Ok(i16::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_i32(&mut self) -> PERes<i32> {
-        Ok(ReadBytesExt::read_i32::<BigEndian>(self)?)
+        let mut buf = [0; 4];
+        self.read_exact(&mut buf)?;
+        Ok(i32::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_i64(&mut self) -> PERes<i64> {
-        Ok(ReadBytesExt::read_i64::<BigEndian>(self)?)
+        let mut buf = [0; 8];
+        self.read_exact(&mut buf)?;
+        Ok(i64::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_i128(&mut self) -> PERes<i128> {
-        Ok(ReadBytesExt::read_i128::<BigEndian>(self)?)
+        let mut buf = [0; 16];
+        self.read_exact(&mut buf)?;
+        Ok(i128::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_f32(&mut self) -> PERes<f32> {
-        Ok(ReadBytesExt::read_f32::<BigEndian>(self)?)
+        let mut buf = [0; 4];
+        self.read_exact(&mut buf)?;
+        Ok(f32::from_be_bytes(buf))
     }
 
     #[inline]
     fn read_f64(&mut self) -> PERes<f64> {
-        Ok(ReadBytesExt::read_f64::<BigEndian>(self)?)
+        let mut buf = [0; 8];
+        self.read_exact(&mut buf)?;
+        Ok(f64::from_be_bytes(buf))
     }
 }
 
